import Content from 'components/Content/Content'
import CozyBar from 'components/Header/CozyBar'
import Header from 'components/Header/Header'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { Ecogesture } from 'models'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import { useNavigate } from 'react-router-dom'
import EcogestureService from 'services/ecogesture.service'
import { useAppSelector } from 'store/hooks'
import EcogestureSelectionDetail from './EcogestureSelectionDetail/EcogestureSelectionDetail'
import EcogestureSelectionEnd from './EcogestureSelectionEnd/EcogestureSelectionEnd'
import EcogestureSelectionModal from './EcogestureSelectionModal/EcogestureSelectionModal'
import EcogestureSelectionRestart from './EcogestureSelectionRestart/EcogestureSelectionRestart'
import './ecogestureSelectionView.scss'

const EcogestureSelectionView = () => {
  const { t } = useI18n()
  const client = useClient()
  const navigate = useNavigate()
  const {
    profileType,
    profileEcogesture,
    profile: { isProfileTypeCompleted },
  } = useAppSelector(state => state.ecolyo)
  const [isLoading, setIsLoading] = useState(true)
  const [indexEcogesture, setIndexEcogesture] = useState<number>(0)
  const [ecogestureList, setEcogestureList] = useState<Ecogesture[]>([])
  const [totalViewed, setTotalViewed] = useState<number>(0)
  const [totalAvailable, setTotalAvailable] = useState<number>(0)
  const [openEcSelectionModal, setOpenEcSelectionModal] = useState(false)

  const ecogestureService = useMemo(
    () => new EcogestureService(client),
    [client]
  )

  const getTitle = useCallback((): string => {
    if (
      indexEcogesture <= ecogestureList.length - 1 &&
      ecogestureList[indexEcogesture]
    ) {
      return ecogestureList[indexEcogesture].shortName
    }
    return t('ecogesture_selection.header')
  }, [ecogestureList, indexEcogesture, t])

  const validateChoice = useCallback(
    async (objective: boolean, doing: boolean) => {
      const updatedEcogesture = await ecogestureService.updateEcogesture({
        ...ecogestureList[indexEcogesture],
        objective: objective,
        doing: doing,
        viewedInSelection: true,
      })
      setEcogestureList(prevList => {
        const updatedList = [...prevList]
        updatedList[indexEcogesture] = updatedEcogesture
        return updatedList
      })
      setIndexEcogesture(prev => prev + 1)
    },
    [ecogestureList, ecogestureService, indexEcogesture]
  )

  const restartSelection = useCallback(async () => {
    setIsLoading(true)
    const profile = isProfileTypeCompleted ? profileType : profileEcogesture
    const availableList =
      await ecogestureService.getEcogestureListByProfile(profile)
    const filteredList = availableList.filter(
      ecogesture => ecogesture.viewedInSelection === false
    )
    const slicedFilteredList = filteredList.slice(0, 10)
    setTotalViewed(availableList.length - filteredList.length)
    setEcogestureList(slicedFilteredList)
    setIndexEcogesture(0)
    setIsLoading(false)
  }, [
    ecogestureService,
    isProfileTypeCompleted,
    profileEcogesture,
    profileType,
  ])

  useEffect(() => {
    let subscribed = true
    async function getFilteredList() {
      const profile = isProfileTypeCompleted ? profileType : profileEcogesture
      const availableList =
        await ecogestureService.getEcogestureListByProfile(profile)
      const filteredList = availableList.filter(
        ecogesture => ecogesture.viewedInSelection === false
      )
      const slicedFilteredList = filteredList.slice(0, 10)
      if (subscribed) {
        if (
          availableList.length === slicedFilteredList.length &&
          slicedFilteredList.length > 0
        ) {
          setOpenEcSelectionModal(true)
        }
        setTotalAvailable(availableList.length)
        setTotalViewed(availableList.length - filteredList.length)
        setEcogestureList(slicedFilteredList)
        setIndexEcogesture(0)
        setIsLoading(false)
      }
    }

    getFilteredList()
    return () => {
      subscribed = false
    }
  }, [
    ecogestureService,
    profileType,
    profileEcogesture,
    isProfileTypeCompleted,
  ])

  const renderEcogestureSelection = () => {
    if (indexEcogesture <= ecogestureList.length - 1) {
      return (
        <EcogestureSelectionDetail
          ecogesture={ecogestureList[indexEcogesture]}
          validate={validateChoice}
          title={getTitle()}
        />
      )
    } else if (totalAvailable > totalViewed + ecogestureList.length) {
      return (
        <EcogestureSelectionRestart
          listLength={ecogestureList.length}
          restart={restartSelection}
        />
      )
    } else {
      return <EcogestureSelectionEnd />
    }
  }

  return (
    <>
      <CozyBar
        titleKey="common.title_ecogestures_choice"
        displayBackArrow={true}
        backFunction={() => navigate('/ecogestures')}
      />
      <Header
        desktopTitleKey="common.title_ecogestures_choice"
        displayBackArrow={true}
      >
        <div className="eg-selection-header">
          {indexEcogesture <= ecogestureList.length - 1
            ? `${totalViewed + indexEcogesture + 1}/${totalAvailable}`
            : ''}
        </div>
      </Header>
      <Content>
        {isLoading && (
          <div className="loaderContainer">
            <Loader />
          </div>
        )}
        {!isLoading && (
          <>
            {renderEcogestureSelection()}
            {openEcSelectionModal && (
              <EcogestureSelectionModal
                open={openEcSelectionModal}
                handleCloseClick={() => setOpenEcSelectionModal(false)}
              />
            )}
          </>
        )}
      </Content>
    </>
  )
}

export default EcogestureSelectionView
