/* eslint-disable camelcase */
import { FluidSlugType, FluidState, FluidType } from 'enums'
import { DateTime } from 'luxon'
import { FluidStatus } from 'models'
import { accountsData } from 'tests/__mocks__/accountsData.mock'
import mockClient from 'tests/__mocks__/client.mock'
import { konnectorsData } from 'tests/__mocks__/konnectorsData.mock'
import { triggerStateData } from 'tests/__mocks__/triggerStateData.mock'
import { triggersData } from 'tests/__mocks__/triggersData.mock'
import FluidService from './fluid.service'

const mockGetAccountByType = jest.fn()
jest.mock('./account.service', () => {
  return jest.fn(() => ({
    getAccountByType: mockGetAccountByType,
  }))
})

const mockGetKonnector = jest.fn()
jest.mock('./konnector.service', () => {
  return jest.fn(() => ({
    getKonnector: mockGetKonnector,
  }))
})

const mockGetTrigger = jest.fn()
const mockFetchTriggerState = jest.fn()
jest.mock('./triggers.service', () => {
  return jest.fn(() => ({
    getTrigger: mockGetTrigger,
    fetchTriggerState: mockFetchTriggerState,
  }))
})

const mockFetchAllFirstDateData = jest.fn()
const mockFetchAllLastDateData = jest.fn()
jest.mock('./consumption.service', () => {
  return jest.fn(() => ({
    fetchAllFirstDateData: mockFetchAllFirstDateData,
    fetchAllLastDateData: mockFetchAllLastDateData,
  }))
})

const mockDataDates: (DateTime | null)[] = [
  DateTime.local().setZone('utc', {
    keepLocalTime: true,
  }),
  DateTime.local().setZone('utc', {
    keepLocalTime: true,
  }),
  DateTime.local().setZone('utc', {
    keepLocalTime: true,
  }),
]

describe('Fluid service', () => {
  const fluidService = new FluidService(mockClient)

  beforeEach(() => {
    mockGetAccountByType.mockReset()
    mockGetTrigger.mockReset()
    mockFetchTriggerState.mockReset()
    mockFetchTriggerState.mockReset()
    mockFetchAllFirstDateData.mockReset()
    mockFetchAllLastDateData.mockReset()
  })

  describe('getFluidStatus method', () => {
    it('should return fluid status for all fluids', async () => {
      const mockResult: FluidStatus[] = [
        {
          fluidType: FluidType.ELECTRICITY,
          status: FluidState.ERROR,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.ELECTRICITY],
          lastDataDate: mockDataDates[FluidType.ELECTRICITY],
          connection: {
            konnector: konnectorsData[0],
            account: accountsData[0],
            trigger: triggersData[0],
            triggerState: triggerStateData,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Enedis',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'https://mon-compte-client.enedis.fr/',
              activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
            },
          },
        },
        {
          fluidType: FluidType.WATER,
          status: FluidState.ERROR,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.WATER],
          lastDataDate: mockDataDates[FluidType.WATER],
          connection: {
            konnector: konnectorsData[1],
            account: accountsData[1],
            trigger: triggersData[1],
            triggerState: triggerStateData,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Eau Publique du Grand Lyon',
              oauth: false,
              slug: FluidSlugType.WATER,
              siteLink:
                'https://www.eaudugrandlyon.com/inscription.aspx#subc-now',
              activation: '',
            },
          },
        },
        {
          fluidType: FluidType.GAS,
          status: FluidState.ERROR,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.GAS],
          lastDataDate: mockDataDates[FluidType.GAS],
          connection: {
            konnector: konnectorsData[2],
            account: accountsData[2],
            trigger: triggersData[2],
            triggerState: triggerStateData,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'GRDF',
              oauth: false,
              slug: FluidSlugType.GAS,
              siteLink: 'https://monespace.grdf.fr/creation-particulier',
              activation: '',
            },
          },
        },
      ]
      mockGetAccountByType
        .mockResolvedValueOnce(accountsData[0])
        .mockResolvedValueOnce(accountsData[1])
        .mockResolvedValueOnce(accountsData[2])
      mockGetKonnector
        .mockResolvedValueOnce(konnectorsData[0])
        .mockResolvedValueOnce(konnectorsData[1])
        .mockResolvedValueOnce(konnectorsData[2])
      mockGetTrigger
        .mockResolvedValueOnce(triggersData[0])
        .mockResolvedValueOnce(triggersData[1])
        .mockResolvedValueOnce(triggersData[2])
      mockFetchTriggerState.mockResolvedValue(triggerStateData)
      mockFetchAllFirstDateData.mockResolvedValue(mockDataDates)
      mockFetchAllLastDateData.mockResolvedValue(mockDataDates)
      const result: FluidStatus[] = await fluidService.getFluidStatus()
      expect(result).toEqual(mockResult)
    })

    it('should return fluid status with NOT_CONNECTED status when no accounts', async () => {
      const mockResult: FluidStatus[] = [
        {
          fluidType: FluidType.ELECTRICITY,
          status: FluidState.NOT_CONNECTED,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.ELECTRICITY],
          lastDataDate: mockDataDates[FluidType.ELECTRICITY],
          connection: {
            konnector: konnectorsData[0],
            account: null,
            trigger: null,
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Enedis',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'https://mon-compte-client.enedis.fr/',
              activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
            },
          },
        },
        {
          fluidType: FluidType.WATER,
          status: FluidState.NOT_CONNECTED,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.WATER],
          lastDataDate: mockDataDates[FluidType.WATER],
          connection: {
            konnector: konnectorsData[1],
            account: null,
            trigger: null,
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Eau Publique du Grand Lyon',
              oauth: false,
              slug: FluidSlugType.WATER,
              siteLink:
                'https://www.eaudugrandlyon.com/inscription.aspx#subc-now',
              activation: '',
            },
          },
        },
        {
          fluidType: FluidType.GAS,
          status: FluidState.NOT_CONNECTED,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.GAS],
          lastDataDate: mockDataDates[FluidType.GAS],
          connection: {
            konnector: konnectorsData[2],
            account: null,
            trigger: null,
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'GRDF',
              oauth: false,
              slug: FluidSlugType.GAS,
              siteLink: 'https://monespace.grdf.fr/creation-particulier',
              activation: '',
            },
          },
        },
      ]
      mockGetAccountByType
        .mockResolvedValueOnce(null)
        .mockResolvedValueOnce(null)
        .mockResolvedValueOnce(null)
      mockGetKonnector
        .mockResolvedValueOnce(konnectorsData[0])
        .mockResolvedValueOnce(konnectorsData[1])
        .mockResolvedValueOnce(konnectorsData[2])
      mockGetTrigger
        .mockResolvedValueOnce(triggersData[0])
        .mockResolvedValueOnce(triggersData[1])
        .mockResolvedValueOnce(triggersData[2])
      mockFetchTriggerState.mockResolvedValue(triggerStateData)
      mockFetchAllFirstDateData.mockResolvedValue(mockDataDates)
      mockFetchAllLastDateData.mockResolvedValue(mockDataDates)
      const result: FluidStatus[] = await fluidService.getFluidStatus()
      expect(result).toEqual(mockResult)
    })

    it('should return fluid status with KONNECTOR_NOT_FOUND status when no konnector', async () => {
      const mockResult: FluidStatus[] = [
        {
          fluidType: FluidType.ELECTRICITY,
          status: FluidState.KONNECTOR_NOT_FOUND,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.ELECTRICITY],
          lastDataDate: mockDataDates[FluidType.ELECTRICITY],
          connection: {
            konnector: null,
            account: accountsData[0],
            trigger: null,
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Enedis',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'https://mon-compte-client.enedis.fr/',
              activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
            },
          },
        },
        {
          fluidType: FluidType.WATER,
          status: FluidState.KONNECTOR_NOT_FOUND,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.WATER],
          lastDataDate: mockDataDates[FluidType.WATER],
          connection: {
            konnector: null,
            account: accountsData[1],
            trigger: null,
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Eau Publique du Grand Lyon',
              oauth: false,
              slug: FluidSlugType.WATER,
              siteLink:
                'https://www.eaudugrandlyon.com/inscription.aspx#subc-now',
              activation: '',
            },
          },
        },
        {
          fluidType: FluidType.GAS,
          status: FluidState.KONNECTOR_NOT_FOUND,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.GAS],
          lastDataDate: mockDataDates[FluidType.GAS],
          connection: {
            konnector: null,
            account: accountsData[2],
            trigger: null,
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'GRDF',
              oauth: false,
              slug: FluidSlugType.GAS,
              siteLink: 'https://monespace.grdf.fr/creation-particulier',
              activation: '',
            },
          },
        },
      ]
      mockGetAccountByType
        .mockResolvedValueOnce(accountsData[0])
        .mockResolvedValueOnce(accountsData[1])
        .mockResolvedValueOnce(accountsData[2])
      mockGetKonnector
        .mockResolvedValueOnce(null)
        .mockResolvedValueOnce(null)
        .mockResolvedValueOnce(null)
      mockGetTrigger
        .mockResolvedValueOnce(triggersData[0])
        .mockResolvedValueOnce(triggersData[1])
        .mockResolvedValueOnce(triggersData[2])
      mockFetchTriggerState.mockResolvedValue(triggerStateData)
      mockFetchAllFirstDateData.mockResolvedValue(mockDataDates)
      mockFetchAllLastDateData.mockResolvedValue(mockDataDates)
      const result: FluidStatus[] = await fluidService.getFluidStatus()
      expect(result).toEqual(mockResult)
    })

    it('should return fluid status with NOT_CONNECTED status when no triggers', async () => {
      const mockResult: FluidStatus[] = [
        {
          fluidType: FluidType.ELECTRICITY,
          status: FluidState.NOT_CONNECTED,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.ELECTRICITY],
          lastDataDate: mockDataDates[FluidType.ELECTRICITY],
          connection: {
            konnector: konnectorsData[0],
            account: accountsData[0],
            trigger: null,
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Enedis',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'https://mon-compte-client.enedis.fr/',
              activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
            },
          },
        },
        {
          fluidType: FluidType.WATER,
          status: FluidState.NOT_CONNECTED,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.WATER],
          lastDataDate: mockDataDates[FluidType.WATER],
          connection: {
            konnector: konnectorsData[1],
            account: accountsData[1],
            trigger: null,
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Eau Publique du Grand Lyon',
              oauth: false,
              slug: FluidSlugType.WATER,
              siteLink:
                'https://www.eaudugrandlyon.com/inscription.aspx#subc-now',
              activation: '',
            },
          },
        },
        {
          fluidType: FluidType.GAS,
          status: FluidState.NOT_CONNECTED,
          maintenance: false,
          firstDataDate: mockDataDates[FluidType.GAS],
          lastDataDate: mockDataDates[FluidType.GAS],
          connection: {
            konnector: konnectorsData[2],
            account: accountsData[2],
            trigger: null,
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'GRDF',
              oauth: false,
              slug: FluidSlugType.GAS,
              siteLink: 'https://monespace.grdf.fr/creation-particulier',
              activation: '',
            },
          },
        },
      ]
      mockGetAccountByType
        .mockResolvedValueOnce(accountsData[0])
        .mockResolvedValueOnce(accountsData[1])
        .mockResolvedValueOnce(accountsData[2])
      mockGetKonnector
        .mockResolvedValueOnce(konnectorsData[0])
        .mockResolvedValueOnce(konnectorsData[1])
        .mockResolvedValueOnce(konnectorsData[2])
      mockGetTrigger
        .mockResolvedValueOnce(null)
        .mockResolvedValueOnce(null)
        .mockResolvedValueOnce(null)
      mockFetchTriggerState.mockResolvedValue(triggerStateData)
      mockFetchAllFirstDateData.mockResolvedValue(mockDataDates)
      mockFetchAllLastDateData.mockResolvedValue(mockDataDates)
      const result: FluidStatus[] = await fluidService.getFluidStatus()
      expect(result).toEqual(mockResult)
    })

    it('should return fluid status with a null first/last date for water and gaz', async () => {
      const _mockDataDates: (DateTime | null)[] = [DateTime.local(), null, null]
      const mockResult: FluidStatus[] = [
        {
          fluidType: FluidType.ELECTRICITY,
          status: FluidState.ERROR,
          maintenance: false,
          firstDataDate: _mockDataDates[FluidType.ELECTRICITY],
          lastDataDate: _mockDataDates[FluidType.ELECTRICITY],
          connection: {
            konnector: konnectorsData[0],
            account: accountsData[0],
            trigger: triggersData[0],
            triggerState: triggerStateData,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Enedis',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'https://mon-compte-client.enedis.fr/',
              activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
            },
          },
        },
        {
          fluidType: FluidType.WATER,
          status: FluidState.NOT_CONNECTED,
          maintenance: false,
          firstDataDate: null,
          lastDataDate: null,
          connection: {
            konnector: konnectorsData[1],
            account: accountsData[1],
            trigger: triggersData[1],
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Eau Publique du Grand Lyon',
              oauth: false,
              slug: FluidSlugType.WATER,
              siteLink:
                'https://www.eaudugrandlyon.com/inscription.aspx#subc-now',
              activation: '',
            },
          },
        },
        {
          fluidType: FluidType.GAS,
          status: FluidState.NOT_CONNECTED,
          maintenance: false,
          firstDataDate: null,
          lastDataDate: null,
          connection: {
            konnector: konnectorsData[2],
            account: accountsData[2],
            trigger: triggersData[2],
            triggerState: null,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'GRDF',
              oauth: false,
              slug: FluidSlugType.GAS,
              siteLink: 'https://monespace.grdf.fr/creation-particulier',
              activation: '',
            },
          },
        },
      ]
      mockGetAccountByType
        .mockResolvedValueOnce(accountsData[0])
        .mockResolvedValueOnce(accountsData[1])
        .mockResolvedValueOnce(accountsData[2])
      mockGetKonnector
        .mockResolvedValueOnce(konnectorsData[0])
        .mockResolvedValueOnce(konnectorsData[1])
        .mockResolvedValueOnce(konnectorsData[2])
      mockGetTrigger
        .mockResolvedValueOnce(triggersData[0])
        .mockResolvedValueOnce(triggersData[1])
        .mockResolvedValueOnce(triggersData[2])
      mockFetchTriggerState
        .mockResolvedValueOnce(triggerStateData)
        .mockResolvedValueOnce(null)
        .mockResolvedValueOnce(null)
      mockFetchAllFirstDateData.mockResolvedValue(_mockDataDates)
      mockFetchAllLastDateData.mockResolvedValue(_mockDataDates)
      const result: FluidStatus[] = await fluidService.getFluidStatus()
      expect(result).toEqual(mockResult)
    })

    it('should return fluids in maintenance', async () => {
      mockFetchAllFirstDateData.mockResolvedValue(mockDataDates)
      mockFetchAllLastDateData.mockResolvedValue(mockDataDates)
      const fluidStatus = await fluidService.getFluidStatus({
        egl_failure: true,
        enedis_failure: true,
        grdf_failure: true,
        notification_activated: true,
      })
      expect(fluidStatus[0].maintenance).toBeTruthy()
      expect(fluidStatus[1].maintenance).toBeTruthy()
      expect(fluidStatus[2].maintenance).toBeTruthy()
    })
  })

  describe('getOldFluidData method', () => {
    it('should return Electricity as old fluid', async () => {
      const mockFluidStatus: FluidStatus[] = [
        {
          fluidType: FluidType.ELECTRICITY,
          status: FluidState.DONE,
          maintenance: false,
          firstDataDate: DateTime.fromISO('2019-01-01').setZone('utc', {
            keepLocalTime: true,
          }),
          lastDataDate: DateTime.fromISO('2020-01-01').setZone('utc', {
            keepLocalTime: true,
          }),
          connection: {
            konnector: konnectorsData[0],
            account: accountsData[0],
            trigger: triggersData[0],
            triggerState: triggerStateData,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Enedis',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'https://mon-compte-client.enedis.fr/',
              activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
            },
          },
        },
      ]
      const result = await FluidService.getOldFluidData(mockFluidStatus)
      expect(result).toEqual([FluidType.ELECTRICITY])
    })

    it('should return empty array as lastdatadate < 5 days', async () => {
      const mockFluidStatus: FluidStatus[] = [
        {
          fluidType: FluidType.ELECTRICITY,
          status: FluidState.DONE,
          maintenance: false,
          firstDataDate: DateTime.local().minus({ day: 31 }).setZone('utc', {
            keepLocalTime: true,
          }),
          lastDataDate: DateTime.local().minus({ day: 1 }).setZone('utc', {
            keepLocalTime: true,
          }),
          connection: {
            konnector: konnectorsData[0],
            account: accountsData[0],
            trigger: triggersData[0],
            triggerState: triggerStateData,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Enedis',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'https://mon-compte-client.enedis.fr/',
              activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
            },
          },
        },
      ]
      const result = await FluidService.getOldFluidData(mockFluidStatus)
      expect(result).toEqual([])
    })

    it('should return empty array as status is NOT_CONNECTED', async () => {
      const mockFluidStatus: FluidStatus[] = [
        {
          fluidType: FluidType.ELECTRICITY,
          status: FluidState.NOT_CONNECTED,
          maintenance: false,
          firstDataDate: DateTime.fromISO('2019-01-01').setZone('utc', {
            keepLocalTime: true,
          }),
          lastDataDate: DateTime.fromISO('2020-01-01').setZone('utc', {
            keepLocalTime: true,
          }),
          connection: {
            konnector: konnectorsData[0],
            account: accountsData[0],
            trigger: triggersData[0],
            triggerState: triggerStateData,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Enedis',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'https://mon-compte-client.enedis.fr/',
              activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
            },
          },
        },
      ]
      const result: FluidType[] =
        await FluidService.getOldFluidData(mockFluidStatus)
      expect(result).toEqual([])
    })

    it('should return empty array as status is KONNECTOR_NOT_FOUND', async () => {
      const mockFluidStatus: FluidStatus[] = [
        {
          fluidType: FluidType.ELECTRICITY,
          status: FluidState.KONNECTOR_NOT_FOUND,
          maintenance: false,
          firstDataDate: DateTime.fromISO('2019-01-01').setZone('utc', {
            keepLocalTime: true,
          }),
          lastDataDate: DateTime.fromISO('2020-01-01').setZone('utc', {
            keepLocalTime: true,
          }),
          connection: {
            konnector: konnectorsData[0],
            account: accountsData[0],
            trigger: triggersData[0],
            triggerState: triggerStateData,
            shouldLaunchKonnector: false,
            isUpdating: false,
            konnectorConfig: {
              name: 'Enedis',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'https://mon-compte-client.enedis.fr/',
              activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
            },
          },
        },
      ]
      const result: FluidType[] =
        await FluidService.getOldFluidData(mockFluidStatus)
      expect(result).toEqual([])
    })

    it('should return empty array', async () => {
      const result: FluidType[] = await FluidService.getOldFluidData([])
      expect(result).toEqual([])
    })
  })
})
