/* eslint-disable camelcase */
import { FluidSlugType, FluidState, FluidType, ScreenType, Usage } from 'enums'
import { DateTime } from 'luxon'
import { FluidStatus, PartnersInfo, TermsStatus } from 'models'
import { SgeStore } from 'models/sgeStore.model'
import { accountsData } from 'tests/__mocks__/accountsData.mock'
import { konnectorsData } from 'tests/__mocks__/konnectorsData.mock'
import { mockGlobalState } from 'tests/__mocks__/store'
import { triggersData } from 'tests/__mocks__/triggersData.mock'
import {
  changeScreenType,
  globalSlice,
  setFluidStatus,
  setHeaderHeight,
  setLastEpglLogin,
  setPartnersInfo,
  setShouldRefreshConsent,
  showReleaseNotes,
  toggleAnalysisNotification,
  toggleChallengeActionNotification,
  toggleChallengeDuelNotification,
  toggleChallengeExplorationNotification,
  updateEcogestureFilter,
  updateFluidConnection,
  updateSgeStore,
  updateTermsStatus,
} from './global.slice'

const nowUtc = DateTime.local().setZone('utc', { keepLocalTime: true })
const mockDataDates = Array(3).fill(nowUtc)

const fluidStatus: FluidStatus[] = [
  {
    fluidType: FluidType.ELECTRICITY,
    status: FluidState.ERROR,
    maintenance: false,
    firstDataDate: mockDataDates[FluidType.ELECTRICITY],
    lastDataDate: mockDataDates[FluidType.ELECTRICITY],
    connection: {
      konnector: konnectorsData[0],
      account: accountsData[0],
      trigger: triggersData[0],
      triggerState: null,
      shouldLaunchKonnector: false,
      isUpdating: false,
      konnectorConfig: {
        name: 'Enedis',
        oauth: false,
        slug: FluidSlugType.ELECTRICITY,
        siteLink: 'https://mon-compte-particulier.enedis.fr/donnees/',
        activation: 'https://mon-compte-particulier.enedis.fr/donnees/',
      },
    },
  },
  {
    fluidType: FluidType.WATER,
    status: FluidState.ERROR,
    maintenance: false,
    firstDataDate: mockDataDates[FluidType.WATER],
    lastDataDate: mockDataDates[FluidType.WATER],
    connection: {
      konnector: konnectorsData[1],
      account: accountsData[1],
      trigger: triggersData[1],
      triggerState: null,
      shouldLaunchKonnector: false,
      isUpdating: false,
      konnectorConfig: {
        name: 'Eau Publique du Grand Lyon',
        oauth: false,
        slug: FluidSlugType.WATER,
        siteLink: 'https://agence.eaudugrandlyon.com/inscription.aspx#subc-now',
        activation: '',
      },
    },
  },
  {
    fluidType: FluidType.GAS,
    status: FluidState.KONNECTOR_NOT_FOUND,
    maintenance: false,
    firstDataDate: mockDataDates[FluidType.GAS],
    lastDataDate: mockDataDates[FluidType.GAS],
    connection: {
      konnector: null,
      account: accountsData[2],
      trigger: null,
      triggerState: null,
      shouldLaunchKonnector: false,
      isUpdating: false,
      konnectorConfig: {
        name: 'GRDF',
        oauth: false,
        slug: FluidSlugType.GAS,
        siteLink: 'https://monespace.grdf.fr/creation-particulier',
        activation: '',
      },
    },
  },
]

describe('globalSlice', () => {
  it('should return the initial state', () => {
    const initialState = globalSlice.getInitialState()
    expect(initialState).toEqual(mockGlobalState)
  })

  it('should handle changeScreenType', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      changeScreenType(ScreenType.DESKTOP)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      screenType: ScreenType.DESKTOP,
    })
  })
  it('should handle setHeaderHeight', () => {
    const state = globalSlice.reducer(mockGlobalState, setHeaderHeight(100))
    expect(state).toEqual({
      ...mockGlobalState,
      headerHeight: 100,
    })
  })
  it('should handle toggleChallengeExplorationNotification', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      toggleChallengeExplorationNotification(true)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      challengeExplorationNotification: true,
    })
  })
  it('should handle toggleChallengeActionNotification', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      toggleChallengeActionNotification(true)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      challengeActionNotification: true,
    })
  })
  it('should handle toggleChallengeDuelNotification', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      toggleChallengeDuelNotification(true)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      challengeDuelNotification: true,
    })
  })
  it('should handle toggleAnalysisNotification', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      toggleAnalysisNotification(true)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      analysisNotification: true,
    })
  })
  it('should handle setFluidStatus', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      setFluidStatus(fluidStatus)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      fluidStatus: fluidStatus,
      fluidTypes: [FluidType.ELECTRICITY, FluidType.WATER],
    })
  })

  it('should handle updateTermsStatus', () => {
    const expectedTermsStatus: TermsStatus = {
      accepted: false,
      versionType: 'major',
    }
    const state = globalSlice.reducer(
      mockGlobalState,
      updateTermsStatus(expectedTermsStatus)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      termsStatus: expectedTermsStatus,
    })
  })

  it('should handle showReleaseNotes', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      showReleaseNotes({
        show: true,
        notes: [{ description: 'desc', title: 'Title' }],
      })
    )
    expect(state).toEqual({
      ...mockGlobalState,
      releaseNotes: {
        show: true,
        notes: [{ description: 'desc', title: 'Title' }],
      },
    })
  })

  it('should handle setPartnersInfo', () => {
    const expectedPartnersInfo: PartnersInfo = {
      egl_failure: true,
      enedis_failure: true,
      grdf_failure: true,
      notification_activated: true,
    }
    const state = globalSlice.reducer(
      mockGlobalState,
      setPartnersInfo(expectedPartnersInfo)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      partnersInfo: expectedPartnersInfo,
    })
  })

  it('should handle setShouldRefreshConsent', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      setShouldRefreshConsent(true)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      shouldRefreshConsent: true,
    })
  })
  it('should handle setSgeConnect', () => {
    const expectedSgeConnect: SgeStore = {
      address: 'address',
      city: 'city',
      currentStep: 1,
      dataConsent: true,
      firstName: 'firstName',
      lastName: 'lastName',
      pdl: 12345678901234,
      pdlConfirm: true,
      shouldLaunchAccount: true,
      zipCode: 99999,
    }
    const state = globalSlice.reducer(
      mockGlobalState,
      updateSgeStore(expectedSgeConnect)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      sgeConnect: expectedSgeConnect,
    })
  })
  it('should handle updateFluidConnection', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      updateFluidConnection({
        fluidType: FluidType.ELECTRICITY,
        fluidConnection: {
          shouldLaunchKonnector: true,
          isUpdating: true,
          konnector: null,
          account: null,
          trigger: null,
          triggerState: null,
          konnectorConfig: {
            name: 'config',
            oauth: false,
            slug: FluidSlugType.ELECTRICITY,
            siteLink: 'link',
            activation: 'activation',
          },
        },
      })
    )
    expect(state).toEqual({
      ...mockGlobalState,
      fluidStatus: [
        {
          fluidType: FluidType.ELECTRICITY,
          firstDataDate: null,
          lastDataDate: null,
          maintenance: false,
          status: FluidState.KONNECTOR_NOT_FOUND,
          connection: {
            shouldLaunchKonnector: true,
            isUpdating: true,
            konnector: null,
            account: null,
            trigger: null,
            triggerState: null,
            konnectorConfig: {
              name: 'config',
              oauth: false,
              slug: FluidSlugType.ELECTRICITY,
              siteLink: 'link',
              activation: 'activation',
            },
          },
        },
        ...mockGlobalState.fluidStatus.slice(1),
      ],
    })
  })

  it('should handle setLastEpglLogin', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      setLastEpglLogin('1234567')
    )
    expect(state).toEqual({
      ...mockGlobalState,
      lastEpglLogin: '1234567',
    })
  })

  it('should change the ecogestureFilter', () => {
    const state = globalSlice.reducer(
      mockGlobalState,
      updateEcogestureFilter(Usage.AIR_CONDITIONING)
    )
    expect(state).toEqual({
      ...mockGlobalState,
      ecogestureFilter: Usage.AIR_CONDITIONING,
    })
  })
})
