import * as Sentry from '@sentry/react'
import { Client } from 'cozy-client'
import logger from 'cozy-logger'
import {
  EGL_DAY_DOCTYPE,
  ENEDIS_DAY_DOCTYPE,
  GRDF_DAY_DOCTYPE,
  REMOTE_ORG_ECOLYO_AGENT_PRICES,
  REMOTE_ORG_ECOLYO_AGENT_PRICES_REC,
} from 'doctypes'
import { FluidType, TimeStep } from 'enums'
import { DateTime } from 'luxon'
import { DataloadEntity, FluidPrice, TimePeriod } from 'models'
import ConsumptionDataManager from 'services/consumption.service'
import EnvironmentService from 'services/environment.service'
import FluidPricesService from 'services/fluidsPrices.service'
import QueryRunner from 'services/queryRunner.service'
import { runService } from './service'

const logStack = logger.namespace('fluidPrices')

const getRemotePricesByFluid = async (
  client: Client,
  fluidType: FluidType
): Promise<FluidPrice[]> => {
  const env = new EnvironmentService()
  const remoteUrl = env.isProduction()
    ? REMOTE_ORG_ECOLYO_AGENT_PRICES
    : REMOTE_ORG_ECOLYO_AGENT_PRICES_REC
  const prices = await client
    .getStackClient()
    .fetchJSON('GET', `${remoteUrl}?fluidtype=${fluidType}`)
  return prices
}

/**
 * If a price has been updated, set the oldest startDate of the edited price so we can redo aggregation
 */
function updateFirstEditedPrice(
  firstEditedPrice: string | null,
  remotePrice: FluidPrice
): string {
  return firstEditedPrice === null || firstEditedPrice >= remotePrice.startDate
    ? remotePrice.startDate
    : firstEditedPrice
}

/**
 * Synchro the remote prices with database and returns a date where we have to relaunch aggregation if a price has been edited in backoffice
 * @returns {string | null} the oldest startDate
 */
const synchroPricesToUpdate = async (
  client: Client,
  fluidType: FluidType
): Promise<string | null> => {
  const fps = new FluidPricesService(client)
  try {
    const remotePrices = await getRemotePricesByFluid(client, fluidType)
    let firstEditedPrice: string | null = null
    for (const remotePrice of remotePrices) {
      const existingPrice = await fps.checkIfPriceExists(remotePrice)
      if (!existingPrice) {
        logStack('debug', `Price doesn't exist in db, creating a new price`)
        firstEditedPrice = updateFirstEditedPrice(firstEditedPrice, remotePrice)
        await fps.createPrice(remotePrice)
        continue
      }

      if (existingPrice.UpdatedAt >= remotePrice.UpdatedAt) {
        logStack('debug', `Price up to date`)
        continue
      }
      logStack('debug', `Price exists in db but not up to date, updating it`)
      // If a price has been updated, set the oldest startDate of the edited price so we can redo aggregation
      firstEditedPrice = updateFirstEditedPrice(firstEditedPrice, remotePrice)

      await fps.updatePrice(existingPrice, {
        price: remotePrice.price,
        UpdatedAt: remotePrice.UpdatedAt,
        startDate: remotePrice.startDate,
        endDate: remotePrice.endDate,
      })
    }
    return firstEditedPrice
  } catch (error) {
    logStack('error', `Error: ${error}`)
    Sentry.captureException(error)
    return null
  }
}

const getTimePeriod = async (
  timeStep: TimeStep,
  date: DateTime
): Promise<TimePeriod> => {
  switch (timeStep) {
    case TimeStep.HALF_AN_HOUR:
      return {
        startDate: date,
        endDate: date.plus({ day: 1 }).startOf('day'),
      }
    case TimeStep.DAY:
    case TimeStep.MONTH:
      return {
        startDate: date.startOf('month'),
        endDate: date.endOf('month'),
      }
    case TimeStep.YEAR:
      return {
        startDate: date.startOf('year'),
        endDate: date.endOf('year'),
      }
    default:
      logStack('error', 'Unhandled time period')
      Sentry.captureException(
        JSON.stringify({ error: 'Unhandled time period' })
      )
      throw Error('Unhandled time period')
  }
}

const aggregatePrices = async (
  qr: QueryRunner,
  cdm: ConsumptionDataManager,
  firstDate: DateTime,
  today: DateTime,
  fluidType: FluidType
) => {
  const timeSteps = [TimeStep.MONTH, TimeStep.YEAR]
  logStack(
    'debug',
    `Aggregation started for fluid: ${fluidType}, from ${firstDate}  `
  )
  for (const timeStep of timeSteps) {
    let aggregationDate = DateTime.fromObject({
      year: firstDate.year,
      month: firstDate.month,
      day: firstDate.day,
    })
    try {
      do {
        const timePeriod = await getTimePeriod(timeStep, aggregationDate)
        // Get doc for aggregation
        const dayDocuments = await qr.fetchFluidRawDoctype(
          timePeriod,
          TimeStep.DAY,
          fluidType
        )
        const docToUpdate = await qr.fetchFluidRawDoctype(
          timePeriod,
          timeStep,
          fluidType
        )

        if (docToUpdate?.data && dayDocuments?.data) {
          docToUpdate.data[0].price = dayDocuments.data
            .map((item: DataloadEntity) => item.price || 0)
            .reduce((a: number, b: number) => a + b)
        }
        await cdm.saveDocs(docToUpdate?.data)
        // Update date according to timestep
        if (timeStep === TimeStep.YEAR) {
          aggregationDate = aggregationDate.plus({ year: 1 }).startOf('month')
        } else {
          aggregationDate = aggregationDate.plus({ month: 1 }).startOf('month')
        }
      } while (aggregationDate < today)
    } catch (error) {
      logStack('info', `Error : ${error}`)
      Sentry.captureException(error)
    }
  }

  logStack('debug', `Aggregation done`)
}

const getDoctypeTypeByFluid = (fluidType: FluidType): string => {
  if (fluidType === FluidType.ELECTRICITY) {
    return ENEDIS_DAY_DOCTYPE
  }
  if (fluidType === FluidType.GAS) {
    return GRDF_DAY_DOCTYPE
  }
  if (fluidType === FluidType.WATER) {
    return EGL_DAY_DOCTYPE
  }
  logStack('error', 'Unknown FluidType')
  Sentry.captureException({ error: 'Unknown FluidType Doctype' })
  throw new Error()
}

const getTimeStepsByFluid = (fluidType: FluidType): TimeStep[] => {
  if (fluidType === FluidType.ELECTRICITY) {
    return [TimeStep.DAY, TimeStep.HALF_AN_HOUR]
  }
  if (fluidType === FluidType.GAS || fluidType === FluidType.WATER) {
    return [TimeStep.DAY]
  }
  logStack('error', 'Unknown FluidType')
  Sentry.captureException({ error: 'Unknown FluidType' })
  throw new Error()
}

const applyPrices = async (client: Client, fluidType: FluidType) => {
  // If no doctypes exists, do nothing
  const fluidsPricesService = new FluidPricesService(client)
  const cdm = new ConsumptionDataManager(client)
  const qr = new QueryRunner(client)

  // Synchro db prices with remote prices
  const firstEditedPriceDate = await synchroPricesToUpdate(client, fluidType)
  const firstDataDate = await cdm.fetchAllFirstDateData([fluidType])
  const prices = await fluidsPricesService.getAllPrices()

  if (prices.length === 0) {
    logStack('info', 'No fluidesPrices data')
    return
  }
  logStack('debug', 'fluidPrices data found')
  const firstMinuteData = await cdm.getFirstDataDateFromDoctypeWithPrice(
    getDoctypeTypeByFluid(fluidType)
  )

  // If there is data, update hourly data and daily data
  if (
    !firstDataDate?.[0] ||
    (!firstMinuteData && firstEditedPriceDate === null)
  ) {
    logStack('info', `No data found for fluid ${fluidType}`)
    return
  }
  const today = DateTime.now()
  const timeSteps = getTimeStepsByFluid(fluidType)
  let firstDate: DateTime

  if (firstMinuteData && firstEditedPriceDate) {
    // If there is first data without price and a price edited, set the smallest date
    const firstMinuteDataDate = DateTime.fromObject({
      year: firstMinuteData.year,
      month: firstMinuteData.month,
      day: firstMinuteData.day,
    }).setZone('utc', {
      keepLocalTime: true,
    })
    const formattedFirstEditedPrice = DateTime.fromISO(
      firstEditedPriceDate
    ).setZone('utc', {
      keepLocalTime: true,
    })
    // we want to exclude the period with no data if the edited date is smaller than the first data date
    firstDate = DateTime.min(
      DateTime.max(formattedFirstEditedPrice, firstDataDate[0]),
      firstMinuteDataDate
    )
  } else if (firstMinuteData) {
    firstDate = DateTime.fromObject({
      year: firstMinuteData.year,
      month: firstMinuteData.month,
      day: firstMinuteData.day,
    }).setZone('utc', {
      keepLocalTime: true,
    })
  } else if (firstEditedPriceDate) {
    firstDate = DateTime.max(
      DateTime.fromISO(firstEditedPriceDate).setZone('utc', {
        keepLocalTime: true,
      }),
      firstDataDate[0]
    )
  } else {
    firstDate = today
  }

  // Hourly and daily prices
  for (const timeStep of timeSteps) {
    let date = DateTime.local().setZone('utc', {
      keepLocalTime: true,
    })
    Object.assign(date, firstDate)
    try {
      do {
        const priceData = await fluidsPricesService.getPrices(fluidType, date)
        const timePeriod = await getTimePeriod(timeStep, date)
        const data = await qr.fetchFluidRawDoctype(
          timePeriod,
          timeStep,
          fluidType
        )

        // If lastItem has a price, skip this day (in order to save perf)
        const lastItem = data?.data?.[data.data.length - 1]
        if (lastItem && priceData) {
          // if a price has been updated in backoffice re-calculates all price from the firstEditedPriceDate
          data?.data.forEach((element: DataloadEntity) => {
            element.price = element.load * priceData.price
          })
          await cdm.saveDocs(data.data)
        }
        // Update date
        if (timeStep === TimeStep.HALF_AN_HOUR) {
          date = date.plus({ days: 1 })
        } else {
          date = date.plus({ month: 1 }).startOf('month')
        }
      } while (date < today)
    } catch (error) {
      logStack('error', `ERROR : ${error} `)
      Sentry.captureException(error)
    }
  }

  await aggregatePrices(qr, cdm, firstDate, today, fluidType)
}

const processPrices = async ({ client }: { client: Client }) => {
  logStack('info', `Processing electricity data...`)
  await applyPrices(client, FluidType.ELECTRICITY)
  logStack('info', `Electricity data done`)
  logStack('info', `Processing gas data...`)
  await applyPrices(client, FluidType.GAS)
  logStack('info', `Gas data done`)
  logStack('info', `Processing water data...`)
  await applyPrices(client, FluidType.WATER)
  logStack('info', `Water data done`)
  logStack('info', `processPrices done`)
}

runService(processPrices)
